#include <algorithm>
#include <string>
#include <vector>

#include "pics.h"

using std::string;
using std::vector;

#ifdef _MSC_VER
#include "../minmax.h"
#else
using std::max;
#endif

string::size_type width(const vector<string>& v)
{
	string::size_type maxlen = 0;
	for(std::vector<string>::const_iterator iter = v.begin(); iter != v.end(); ++iter)
		maxlen = max(maxlen, (*iter).size());
	return maxlen;
}

vector<string> frame(const vector<string>& v)
{
	vector<string> ret;
	string::size_type maxlen = width(v);
	string border(maxlen + 4, '*');

	// write the top border
	ret.push_back(border);

	// write each interior row, bordered by an asterisk and a space
	for(std::vector<string>::const_iterator iter = v.begin(); iter != v.end(); ++iter) {
		ret.push_back("* " + *iter +
		              string(maxlen - (*iter).size(), ' ') + " *");
	}

	// write the bottom border
	ret.push_back(border);
	return ret;
}

vector<string> vcat(const vector<string>& top,
                    const vector<string>& bottom)
{
	// copy the `top' picture
	vector<string> ret = top;

	// copy entire `bottom' picture
	for (std::vector<string>::const_iterator it = bottom.begin();
	     it != bottom.end(); ++it)
		ret.push_back(*it);

	return ret;
}

vector<string>
hcat(const vector<string>& left, const vector<string>& right)
{
	vector<string> ret;

	// add 1 to leave a space between pictures
	string::size_type width1 = width(left) + 1;

	// indices to look at elements from `left' and `right' respectively
	vector<string>::const_iterator iter_l = left.begin(), iter_r = right.begin();

	// continue until we've seen all rows from both pictures
	while (iter_l != left.end() || iter_r != right.end()) {
		// construct new `string' to hold characters from both pictures
		string s;

		// copy a row from the left-hand side, if there is one
		if (iter_l != left.end())
			s = *(iter_l++);

		// pad to full width
		s += string(width1 - s.size(), ' ');

		// copy a row from the right-hand side, if there is one
		if (iter_r != right.end())
			s += *(iter_r++);

		// add `s' to the picture we're creating
		ret.push_back(s);
	}

	return ret;
}

